@@ -1087,9 +1087,18 @@ Form::Form(XRef *xrefA, Object* acroForm)
       array->getNF(i, &oref);
       if (!oref.isRef()) {
         error(-1, "Direct object in rootFields");
+	obj1.free();
+	oref.free();
         continue;
       }
 
+      if (!obj1.isDict()) {
+        error(-1, "Reference in Fields array to an invalid or non existant object");
+	obj1.free();
+	oref.free();
+	continue;
+      }
+
       if (numFields >= size) {
         size += 16;
         rootFields = (FormField**)greallocn(rootFields,size,sizeof(FormField*));
