@@ -20,12 +20,14 @@
 #ifndef __POPPLER_QT_H__
 #define __POPPLER_QT_H__
 
+#include "poppler-link-qt3.h"
+#include "poppler-page-transition.h"
+
 #include <qcstring.h>
 #include <qdatetime.h>
+#include <qdom.h>
 #include <qpixmap.h>
 
-#include <poppler-page-transition.h>
-
 namespace Poppler {
 
 class Document;
@@ -70,11 +72,15 @@ public:
     unknown,
     Type1,
     Type1C,
+    Type1COT,
     Type3,
     TrueType,
+    TrueTypeOT,
     CIDType0,
     CIDType0C,
-    CIDTrueType
+    CIDType0COT,
+    CIDTrueType,
+    CIDTrueTypeOT
   };
 
   /**
@@ -125,7 +131,7 @@ class Page {
   friend class Document;
   public:
     ~Page();
-    void renderToPixmap(QPixmap **q, int x, int y, int w, int h, double xres, double yres) const;
+    void renderToPixmap(QPixmap **q, int x, int y, int w, int h, double xres, double yres, bool doLinks = false) const;
 
     /**
       This is a convenience function that is equivalent to
@@ -134,7 +140,7 @@ class Page {
 
       \sa renderToImage()
      */
-    void renderToPixmap(QPixmap **q, int x, int y, int w, int h) const;
+    void renderToPixmap(QPixmap **q, int x, int y, int w, int h, bool doLinks = false) const;
 
     /**
       \brief Render the page to a QImage using the Splash renderer
@@ -152,7 +158,7 @@ class Page {
 
      \sa renderToPixmap()
     */
-    QImage renderToImage(double xres = 72.0, double yres = 72.0) const;
+    QImage renderToImage(double xres = 72.0, double yres = 72.0, bool doLinks = false) const;
 
     /**
      * Returns the size of the page in points
@@ -184,6 +190,11 @@ class Page {
     **/
     Orientation orientation() const;
     
+    /**
+      Gets the links of the page once it has been rendered if doLinks was true
+    */
+    QValueList<Link*> links() const;
+
   private:
     Page(const Document *doc, int index);
     PageData *data;
@@ -225,6 +236,17 @@ public:
   bool okToAddNotes() const;
   double getPDFVersion() const;
 
+  bool print(const QString &fileName, QValueList<int> pageList, double hDPI, double vDPI, int rotate);
+
+  // If you are using QPrinter you can get paper size doing
+  // QPrinter dummy(QPrinter::PrinterResolution);
+  // dummy.setFullPage(true);
+  // dummy.setPageSize(thePageSizeYouWant);
+  // QPaintDeviceMetrics metrics(&dummy);
+  // int width = metrics.width();
+  // int height = metrics.height();
+  bool print(const QString &fileName, QValueList<int> pageList, double hDPI, double vDPI, int rotate, int paperWidth, int paperHeight);
+
   /**
     The fonts within the PDF document.
 
@@ -245,6 +267,23 @@ public:
   */
   bool scanForFonts( int numPages, QValueList<FontInfo> *fontList ) const;
 
+  /**
+    Gets the TOC of the Document, it is application responsabiliy to delete
+    it when no longer needed
+
+    * In the tree the tag name is the 'screen' name of the entry. A tag can have
+    * attributes. Here follows the list of tag attributes with meaning:
+    * - Destination: A string description of the referred destination
+    * - DestinationName: A 'named reference' to the viewport that must be converted
+    *      using linkDestination( *destination_name* )
+    * - ExternalFileName: A link to a external filename
+
+     \returns NULL if the Document does not have TOC
+  */
+  QDomDocument *toc() const;
+
+  LinkDestination *linkDestination( const QString &name );
+
   ~Document();
   
 private:
